@@ -4,6 +4,7 @@ use quote::ToTokens;
 use quote::{quote, quote_spanned};
 use std::fmt::{Debug, Formatter};
 use std::ops::Deref;
+use std::str::FromStr;
 use syn::{FnArg, ForeignItemType, LitStr, Pat, PatType, Path, ReturnType, Type};
 
 use self::bridged_option::BridgedOption;
@@ -50,6 +51,7 @@ pub(crate) enum StdLibType {
     Option(BridgedOption),
 }
 
+/// TODO: Add this to `OpaqueForeignType`
 #[derive(Debug, Copy, Clone)]
 pub(crate) enum TypePosition {
     FnArg,
@@ -386,7 +388,8 @@ impl BridgedType {
             let inner = if let Some(declared_ty) = types.get(inner) {
                 declared_ty.to_bridged_type(false, false)
             } else {
-                BridgedType::with_str(inner, types)?
+                let inner: Type = syn::parse2(TokenStream::from_str(inner).unwrap()).unwrap();
+                BridgedType::new_with_type(&inner, types)?
             };
 
             return Some(BridgedType::StdLib(StdLibType::Vec(BuiltInVec {
@@ -395,7 +398,9 @@ impl BridgedType {
         } else if string.starts_with("Option < ") {
             let inner = string.trim_start_matches("Option < ");
             let inner = inner.trim_end_matches(" >");
-            let inner = BridgedType::with_str(inner, types)?;
+
+            let inner: Type = syn::parse2(TokenStream::from_str(inner).unwrap()).unwrap();
+            let inner = BridgedType::new_with_type(&inner, types)?;
 
             return Some(BridgedType::StdLib(StdLibType::Option(BridgedOption {
                 ty: Box::new(inner),
@@ -679,7 +684,17 @@ impl BridgedType {
             BridgedType::Foreign(CustomBridgedType::Opaque(opaque)) => {
                 if opaque.host_lang.is_rust() {
                     if func_host_lang.is_rust() {
-                        opaque.ty.ident.to_string()
+                        let mut class_name = opaque.ty.ident.to_string();
+
+                        if opaque.reference {
+                            class_name += "Ref";
+                        }
+
+                        if opaque.mutable {
+                            class_name += "Mut";
+                        }
+
+                        class_name
                     } else {
                         format!("UnsafeMutableRawPointer")
                     }
@@ -707,7 +722,17 @@ impl BridgedType {
             }
             BridgedType::Foreign(CustomBridgedType::Opaque(opaque)) => {
                 if opaque.host_lang.is_rust() {
-                    opaque.ty.ident.to_string()
+                    let mut ty_name = opaque.ty.ident.to_string();
+
+                    if opaque.reference {
+                        ty_name += "Ref";
+                    }
+
+                    if opaque.mutable {
+                        ty_name += "Mut";
+                    }
+
+                    ty_name
                 } else {
                     if func_host_lang.is_rust() {
                         opaque.ty.ident.to_string()
@@ -872,13 +897,72 @@ impl BridgedType {
                         &quote! {val},
                     );
 
-                    quote! {
-                        if let Some(val) = #expression {
-                            #swift_bridge_path::option::_set_option_return(true);
-                            #val
-                        } else {
-                            #swift_bridge_path::option::_set_option_return(false);
-                            #unused_none_value
+                    match opt.ty.deref() {
+                        BridgedType::StdLib(stdlib_type) => match stdlib_type {
+                            StdLibType::Null
+                            | StdLibType::U8
+                            | StdLibType::I8
+                            | StdLibType::U16
+                            | StdLibType::I16
+                            | StdLibType::U32
+                            | StdLibType::I32
+                            | StdLibType::U64
+                            | StdLibType::I64
+                            | StdLibType::Usize
+                            | StdLibType::Isize
+                            | StdLibType::F32
+                            | StdLibType::F64
+                            | StdLibType::Bool => {
+                                quote! {
+                                    if let Some(val) = #expression {
+                                        #swift_bridge_path::option::_set_option_return(true);
+                                        #val
+                                    } else {
+                                        #swift_bridge_path::option::_set_option_return(false);
+                                        #unused_none_value
+                                    }
+                                }
+                            }
+                            StdLibType::Pointer(_) => {
+                                todo!("Support Option<*const T> and Option<*mut T>")
+                            }
+                            StdLibType::RefSlice(_) => {
+                                todo!("Support Option<&[T]> and Option<&mut [T]>")
+                            }
+                            StdLibType::Str => {
+                                quote! {
+                                    if let Some(val) = #expression {
+                                        #swift_bridge_path::string::RustStr::from_str(val)
+                                    } else {
+                                        #swift_bridge_path::string::RustStr { start: std::ptr::null::<u8>(), len: 0}
+                                    }
+                                }
+                            }
+                            // TODO: No need to use the _set_option_returns since we're passing
+                            //  pointers. We can just use a null pointer to indicate None.
+                            StdLibType::String => {
+                                quote! {
+                                    if let Some(val) = #expression {
+                                        #swift_bridge_path::option::_set_option_return(true);
+                                        #val
+                                    } else {
+                                        #swift_bridge_path::option::_set_option_return(false);
+                                        #unused_none_value
+                                    }
+                                }
+                            }
+                            StdLibType::Vec(_) => {
+                                todo!("Support Option<Vec<T>>")
+                            }
+                            StdLibType::Option(_) => {
+                                todo!("Support Option<Option<T>>")
+                            }
+                        },
+                        BridgedType::Foreign(CustomBridgedType::Shared(_shared_type)) => {
+                            todo!("Support Option<SharedType>")
+                        }
+                        BridgedType::Foreign(CustomBridgedType::Opaque(_opaque_type)) => {
+                            todo!("Support Option<OpaqueType>")
                         }
                     }
                 }
@@ -1069,36 +1153,91 @@ impl BridgedType {
                 }
                 StdLibType::Str => value.to_string(),
                 StdLibType::String => {
-                    format!("RustString(ptr: {}, isOwned: true)", value)
+                    format!("RustString(ptr: {})", value)
                 }
                 StdLibType::Vec(_ty) => {
-                    format!("RustVec(ptr: {}, isOwned: true)", value)
+                    format!("RustVec(ptr: {})", value)
                 }
                 StdLibType::Option(opt) => {
                     let inner_val = opt.convert_ffi_value_to_swift_value(func_host_lang, type_pos);
 
-                    format!(
-                        "let val = {val}; if _get_option_return() {{ return {inner_val}; }} else {{ return nil; }}",
-                        val = value,
-                        inner_val = inner_val
-                    )
+                    match opt.ty.deref() {
+                        BridgedType::StdLib(stdlib_type) => {
+                            match stdlib_type {
+                                StdLibType::Null
+                                | StdLibType::U8
+                                | StdLibType::I8
+                                | StdLibType::U16
+                                | StdLibType::I16
+                                | StdLibType::U32
+                                | StdLibType::I32
+                                | StdLibType::U64
+                                | StdLibType::I64
+                                | StdLibType::Usize
+                                | StdLibType::Isize
+                                | StdLibType::F32
+                                | StdLibType::F64
+                                | StdLibType::Bool => {
+                                    format!(
+                                        "let val = {val}; if _get_option_return() {{ return {inner_val}; }} else {{ return nil; }}",
+                                        val = value,
+                                        inner_val = inner_val
+                                    )
+                                }
+                                StdLibType::Pointer(_) => {
+                                    todo!("Support Option<*const T> and Option<*mut T>")
+                                }
+                                StdLibType::RefSlice(_) => {
+                                    todo!("Support Option<&[T]>")
+                                }
+                                StdLibType::Str => {
+                                    format!(
+                                        "let val = {val}; if val.start != nil {{ return {inner_val}; }} else {{ return nil; }}",
+                                        val = value,
+                                        inner_val = inner_val
+                                    )
+                                }
+                                StdLibType::String => {
+                                    // TODO: Just check if the pointer is null instead of using
+                                    //  _get_option_return()
+                                    format!(
+                                        "let val = {val}; if _get_option_return() {{ return {inner_val}; }} else {{ return nil; }}",
+                                        val = value,
+                                        inner_val = inner_val
+                                    )
+                                }
+                                StdLibType::Vec(_) => {
+                                    todo!("Support Option<Vec<T>>")
+                                }
+                                StdLibType::Option(_) => {
+                                    todo!("Support Option<Option<T>>")
+                                }
+                            }
+                        }
+                        BridgedType::Foreign(CustomBridgedType::Shared(_shared)) => {
+                            todo!("Support Option<SharedType>")
+                        }
+                        BridgedType::Foreign(CustomBridgedType::Opaque(_opaque)) => {
+                            todo!("Support Option<OpaqueType>")
+                        }
+                    }
                 }
             },
             BridgedType::Foreign(CustomBridgedType::Shared(SharedType::Struct(_shared_struct))) => {
                 format!("{}", value)
             }
             BridgedType::Foreign(CustomBridgedType::Opaque(opaque)) => {
-                let ty_name = &opaque.ty.ident;
+                let mut ty_name = opaque.ty.ident.to_string();
+
+                if opaque.reference {
+                    ty_name += "Ref";
+                }
+                if opaque.mutable {
+                    ty_name += "Mut";
+                }
 
                 if opaque.host_lang.is_rust() {
-                    let is_owned = if opaque.reference { "false" } else { "true" };
-
-                    format!(
-                        "{ty_name}(ptr: {value}, isOwned: {is_owned})",
-                        ty_name = ty_name,
-                        value = value,
-                        is_owned = is_owned
-                    )
+                    format!("{ty_name}(ptr: {value})", ty_name = ty_name, value = value,)
                 } else {
                     format!(
                         "Unmanaged<{ty_name}>.fromOpaque({value}.ptr).takeRetainedValue()",
@@ -1194,7 +1333,7 @@ impl BridgedType {
                             format!("{{{}.isOwned = false; return {}.ptr;}}()", value, value)
                         } else {
                             format!(
-                                "{type_name}(ptr: {value}, isOwned: true)",
+                                "{type_name}(ptr: {value})",
                                 type_name = ty_name,
                                 value = value
                             )
@@ -1286,7 +1425,15 @@ impl BridgedType {
                     todo!("Support Option<&[T]>")
                 }
                 StdLibType::Str => {
-                    todo!("Support Option<&str>")
+                    UnusedOptionNoneValue {
+                        rust: quote! {
+                            #swift_bridge_path::string::RustStr {start: std::ptr::null::<u8>(), len: 0}
+                        },
+                        // TODO: Add integration tests:
+                        //  Rust: crates/swift-integration-tests/src/option.rs
+                        //  Swift: OptionTests.swift
+                        swift: "TODO_SWIFT_OPTIONAL_STR_SUPPORT".to_string(),
+                    }
                 }
                 StdLibType::String => {
                     UnusedOptionNoneValue {
