@@ -1,24 +1,31 @@
+use std::collections::HashMap;
+use std::ops::Deref;
+
+use quote::ToTokens;
+use syn::{Path, ReturnType, Type};
+
 use crate::bridged_type::{fn_arg_name, BridgedType, StdLibType, StructSwiftRepr, TypePosition};
 use crate::codegen::generate_swift::vec::generate_vectorizable_extension;
+use crate::codegen::CodegenConfig;
 use crate::parse::{
     HostLang, OpaqueForeignTypeDeclaration, SharedTypeDeclaration, TypeDeclaration,
     TypeDeclarations,
 };
 use crate::parsed_extern_fn::ParsedExternFn;
 use crate::{SwiftBridgeModule, SWIFT_BRIDGE_PREFIX};
-use quote::ToTokens;
-use std::collections::HashMap;
-use std::ops::Deref;
-use syn::{Path, ReturnType, Type};
 
 mod option;
 mod vec;
 
 impl SwiftBridgeModule {
     /// Generate the corresponding Swift code for the bridging module.
-    pub fn generate_swift(&self) -> String {
+    pub(crate) fn generate_swift(&self, config: &CodegenConfig) -> String {
         let mut swift = "".to_string();
 
+        if !self.module_will_be_compiled(config) {
+            return swift;
+        }
+
         let mut associated_funcs_and_methods: HashMap<String, Vec<&ParsedExternFn>> =
             HashMap::new();
 
@@ -81,8 +88,10 @@ impl SwiftBridgeModule {
                         );
                         swift += "\n";
 
-                        swift += &generate_vectorizable_extension(&ty.ident);
-                        swift += "\n";
+                        if !ty.already_declared {
+                            swift += &generate_vectorizable_extension(&ty.ident);
+                            swift += "\n";
+                        }
                     }
                     HostLang::Swift => {
                         swift += &generate_drop_swift_instance_reference_count(ty);
@@ -110,10 +119,6 @@ fn generate_swift_class(
     let mut ref_self_methods = vec![];
     let mut ref_mut_self_methods = vec![];
 
-    let default_init = r#"    init() {
-        fatalError("No #[swift_bridge(constructor)] was defined in the extern Rust module.")
-    }"#;
-
     if let Some(methods) = associated_funcs_and_methods.get(&type_name) {
         for type_method in methods {
             // TODO: Normalize with freestanding func codegen above
@@ -140,36 +145,15 @@ fn generate_swift_class(
         }
     }
 
-    if initializers.len() == 0 {
-        initializers.push(default_init.to_string());
-    }
+    let class_decl = if ty.already_declared {
+        "".to_string()
+    } else {
+        let free_func_call = format!("{}${}$_free(ptr)", SWIFT_BRIDGE_PREFIX, type_name);
 
-    let initializers: String = initializers.join("\n\n");
-
-    let mut owned_instance_methods: String = owned_self_methods.join("\n\n");
-    if owned_instance_methods.len() > 0 {
-        owned_instance_methods = format!("\n\n{}", owned_instance_methods);
-    }
-
-    let mut ref_instance_methods: String = ref_self_methods.join("\n\n");
-    if ref_instance_methods.len() > 0 {
-        ref_instance_methods = format!("\n\n{}", ref_instance_methods);
-    }
-
-    let mut ref_mut_instance_methods: String = ref_mut_self_methods.join("\n\n");
-    if ref_mut_instance_methods.len() > 0 {
-        ref_mut_instance_methods = format!("\n\n{}", ref_mut_instance_methods);
-    }
-
-    let free_func_call = format!("{}${}$_free(ptr)", SWIFT_BRIDGE_PREFIX, type_name);
-
-    let class = format!(
-        r#"
-public class {type_name}: {type_name}RefMut {{
+        format!(
+            r#"public class {type_name}: {type_name}RefMut {{
     var isOwned: Bool = true
 
-{initializers}
-
     override init(ptr: UnsafeMutableRawPointer) {{
         super.init(ptr: ptr)
     }}
@@ -178,26 +162,107 @@ public class {type_name}: {type_name}RefMut {{
         if isOwned {{
             {free_func_call}
         }}
-    }}{owned_instance_methods}
-}}
+    }}
+}}"#,
+            type_name = type_name,
+            free_func_call = free_func_call
+        )
+    };
+    let class_ref_decl = if ty.already_declared {
+        "".to_string()
+    } else {
+        format!(
+            r#"
 public class {type_name}RefMut: {type_name}Ref {{
     override init(ptr: UnsafeMutableRawPointer) {{
         super.init(ptr: ptr)
-    }}{ref_mut_instance_methods}
-}}
+    }}
+}}"#,
+            type_name = type_name
+        )
+    };
+    let class_ref_mut_decl = if ty.already_declared {
+        "".to_string()
+    } else {
+        format!(
+            r#"
 public class {type_name}Ref {{
     var ptr: UnsafeMutableRawPointer
 
     init(ptr: UnsafeMutableRawPointer) {{
         self.ptr = ptr
-    }}{ref_instance_methods}
+    }}
 }}"#,
-        type_name = type_name,
+            type_name = type_name,
+        )
+    };
+
+    let initializers = if initializers.len() == 0 {
+        "".to_string()
+    } else {
+        let initializers: String = initializers.join("\n\n");
+        format!(
+            r#"
+extension {type_name} {{
+{initializers}
+}}"#,
+            type_name = type_name,
+            initializers = initializers
+        )
+    };
+
+    let owned_instance_methods = if owned_self_methods.len() == 0 {
+        "".to_string()
+    } else {
+        let owned_instance_methods: String = owned_self_methods.join("\n\n");
+        format!(
+            r#"
+extension {type_name} {{
+{owned_instance_methods}
+}}"#,
+            type_name = type_name,
+            owned_instance_methods = owned_instance_methods
+        )
+    };
+
+    let ref_instance_methods = if ref_self_methods.len() == 0 {
+        "".to_string()
+    } else {
+        let ref_instance_methods: String = ref_self_methods.join("\n\n");
+        format!(
+            r#"
+extension {type_name}Ref {{
+{ref_instance_methods}
+}}"#,
+            type_name = type_name,
+            ref_instance_methods = ref_instance_methods
+        )
+    };
+
+    let ref_mut_instance_methods = if ref_mut_self_methods.len() == 0 {
+        "".to_string()
+    } else {
+        let ref_mut_instance_methods: String = ref_mut_self_methods.join("\n\n");
+        format!(
+            r#"
+extension {type_name}RefMut {{
+{ref_mut_instance_methods}
+}}"#,
+            type_name = type_name,
+            ref_mut_instance_methods = ref_mut_instance_methods
+        )
+    };
+
+    let class = format!(
+        r#"
+{class_decl}{initializers}{owned_instance_methods}{class_ref_decl}{ref_mut_instance_methods}{class_ref_mut_decl}{ref_instance_methods}"#,
+        class_decl = class_decl,
+        class_ref_decl = class_ref_decl,
+        class_ref_mut_decl = class_ref_mut_decl,
         initializers = initializers,
         owned_instance_methods = owned_instance_methods,
         ref_mut_instance_methods = ref_mut_instance_methods,
         ref_instance_methods = ref_instance_methods,
-        free_func_call = free_func_call
     );
 
     return class;
@@ -263,7 +328,7 @@ fn gen_func_swift_calls_rust(
     };
 
     let swift_class_func_name = if function.is_initializer {
-        "init".to_string()
+        "convenience init".to_string()
     } else {
         format!("func {}", fn_name.as_str())
     };
@@ -355,7 +420,7 @@ fn gen_func_swift_calls_rust(
     }
 
     if function.is_initializer {
-        call_rust = format!("super.init(ptr: {})", call_rust)
+        call_rust = format!("self.init(ptr: {})", call_rust)
     }
 
     let maybe_return = if function.is_initializer {
@@ -454,11 +519,13 @@ mod tests {
     //! TODO: We're progressively moving most of these tests to `codegen_tests.rs`,
     //!  along with their corresponding Rust token and C header generation tests.
 
-    use crate::test_utils::assert_trimmed_generated_contains_trimmed_expected;
-    use crate::SwiftBridgeModule;
+    use crate::codegen::generate_swift::CodegenConfig;
     use quote::quote;
     use syn::parse_quote;
 
+    use crate::test_utils::assert_trimmed_generated_contains_trimmed_expected;
+    use crate::SwiftBridgeModule;
+
     /// Verify that we generated a Swift function to call our freestanding function.
     #[test]
     fn freestanding_rust_function_no_args() {
@@ -470,7 +537,7 @@ mod tests {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func foo() {
@@ -492,7 +559,7 @@ func foo() {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$foo")
@@ -515,7 +582,7 @@ func __swift_bridge__foo () {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$foo")
@@ -539,7 +606,7 @@ func __swift_bridge__foo () -> __private__FfiSlice {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$MyType$foo")
@@ -562,7 +629,7 @@ func __swift_bridge__MyType_foo (_ this: UnsafeMutableRawPointer) -> __private__
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func foo(_ bar: UInt8) {
@@ -585,7 +652,7 @@ func foo(_ bar: UInt8) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func foo() -> UInt32 {
@@ -607,7 +674,7 @@ func foo() -> UInt32 {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func foo() -> UnsafeBufferPointer<UInt8> {
@@ -630,7 +697,7 @@ func foo() -> UnsafeBufferPointer<UInt8> {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$Foo$_free")
@@ -642,6 +709,33 @@ func __swift_bridge__Foo__free (ptr: UnsafeMutableRawPointer) {
         assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
     }
 
+    /// Verify that we generated a function that Rust can use to reduce a Swift class instance's
+    /// reference count.
+    #[test]
+    fn extern_swift_claas_init() {
+        let tokens = quote! {
+            mod foo {
+                extern "Swift" {
+                    type Foo;
+
+                    #[swift_bridge(init)]
+                    fn new (a: u8) -> Foo;
+                }
+            }
+        };
+        let module: SwiftBridgeModule = parse_quote!(#tokens);
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
+
+        let expected = r#"
+@_cdecl("__swift_bridge__$Foo$new")
+func __swift_bridge__Foo_new (_ a: UInt8) -> __private__PointerToSwiftType {
+    __private__PointerToSwiftType(ptr: Unmanaged.passRetained(Foo(a: a)).toOpaque())
+}
+"#;
+
+        assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
+    }
+
     /// Verify that we generated a Swift class with an init method.
     #[test]
     fn class_with_init() {
@@ -656,16 +750,12 @@ func __swift_bridge__Foo__free (ptr: UnsafeMutableRawPointer) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 public class Foo: FooRefMut {
     var isOwned: Bool = true
 
-    init() {
-        super.init(ptr: __swift_bridge__$Foo$new())
-    }
-
     override init(ptr: UnsafeMutableRawPointer) {
         super.init(ptr: ptr)
     }
@@ -676,36 +766,14 @@ public class Foo: FooRefMut {
         }
     }
 }
-"#;
-
-        assert_trimmed_generated_contains_trimmed_expected(&generated, expected);
+extension Foo {
+    convenience init() {
+        self.init(ptr: __swift_bridge__$Foo$new())
     }
-
-    /// Verify that we generated a function that Rust can use to reduce a Swift class instance's
-    /// reference count.
-    #[test]
-    fn extern_swift_claas_init() {
-        let tokens = quote! {
-            mod foo {
-                extern "Swift" {
-                    type Foo;
-
-                    #[swift_bridge(init)]
-                    fn new (a: u8) -> Foo;
-                }
-            }
-        };
-        let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
-
-        let expected = r#"
-@_cdecl("__swift_bridge__$Foo$new")
-func __swift_bridge__Foo_new (_ a: UInt8) -> __private__PointerToSwiftType {
-    __private__PointerToSwiftType(ptr: Unmanaged.passRetained(Foo(a: a)).toOpaque())
 }
 "#;
 
-        assert_trimmed_generated_contains_trimmed_expected(&generated, &expected);
+        assert_trimmed_generated_contains_trimmed_expected(&generated, expected);
     }
 
     /// Verify that we generated a Swift class with an init method with params.
@@ -722,16 +790,12 @@ func __swift_bridge__Foo_new (_ a: UInt8) -> __private__PointerToSwiftType {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 public class Foo: FooRefMut {
     var isOwned: Bool = true
 
-    init(_ val: UInt8) {
-        super.init(ptr: __swift_bridge__$Foo$new(val))
-    }
-
     override init(ptr: UnsafeMutableRawPointer) {
         super.init(ptr: ptr)
     }
@@ -742,6 +806,11 @@ public class Foo: FooRefMut {
         }
     }
 }
+extension Foo {
+    convenience init(_ val: UInt8) {
+        self.init(ptr: __swift_bridge__$Foo$new(val))
+    }
+}
 "#;
 
         assert_trimmed_generated_contains_trimmed_expected(&generated, expected);
@@ -762,7 +831,7 @@ public class Foo: FooRefMut {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$Foo$push")
@@ -792,7 +861,7 @@ func __swift_bridge__Foo_pop (_ this: UnsafeMutableRawPointer) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 public class FooRef {
@@ -801,7 +870,8 @@ public class FooRef {
     init(ptr: UnsafeMutableRawPointer) {
         self.ptr = ptr
     }
-
+}
+extension FooRef {
     func bar() -> UInt8 {
         __swift_bridge__$Foo$bar(ptr)
     }
@@ -824,7 +894,7 @@ public class FooRef {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 public class FooRef {
@@ -833,7 +903,8 @@ public class FooRef {
     init(ptr: UnsafeMutableRawPointer) {
         self.ptr = ptr
     }
-
+}
+extension FooRef {
     func bar(_ other: FooRef) {
         __swift_bridge__$Foo$bar(ptr, other.ptr)
     }
@@ -857,7 +928,7 @@ public class FooRef {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 public class FooRef {
@@ -866,7 +937,8 @@ public class FooRef {
     init(ptr: UnsafeMutableRawPointer) {
         self.ptr = ptr
     }
-
+}
+extension FooRef {
     class func bar() {
         __swift_bridge__$Foo$bar()
     }
@@ -891,7 +963,7 @@ public class FooRef {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$Foo$bar")
@@ -914,7 +986,7 @@ func __swift_bridge__Foo_bar (_ arg: UInt8) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func foo() -> RustString {
@@ -937,7 +1009,7 @@ func foo() -> RustString {
             }
         };
         let module: SwiftBridgeModule = syn::parse2(start).unwrap();
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func void_pointer(_ arg1: UnsafeRawPointer) {
@@ -960,7 +1032,7 @@ func void_pointer(_ arg1: UnsafeRawPointer) {
             }
         };
         let module: SwiftBridgeModule = syn::parse2(start).unwrap();
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func void_pointer() -> UnsafeRawPointer {
@@ -983,7 +1055,7 @@ func void_pointer() -> UnsafeRawPointer {
             }
         };
         let module: SwiftBridgeModule = syn::parse2(start).unwrap();
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$void_pointer")
@@ -1008,7 +1080,7 @@ func __swift_bridge__void_pointer (_ arg: UnsafeRawPointer) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func get_foo() -> Foo {
@@ -1032,7 +1104,7 @@ func get_foo() -> Foo {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func some_function(_ arg: Foo) {
@@ -1058,7 +1130,7 @@ func some_function(_ arg: Foo) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func some_function(_ arg: Renamed) -> Renamed {
@@ -1085,7 +1157,7 @@ func some_function(_ arg: Renamed) -> Renamed {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func some_function(_ arg: Foo) {
@@ -1112,7 +1184,7 @@ func some_function(_ arg: Foo) {
             }
         };
         let module: SwiftBridgeModule = parse_quote!(#tokens);
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 func some_function() -> Foo {
@@ -1136,7 +1208,7 @@ func some_function() -> Foo {
             }
         };
         let module: SwiftBridgeModule = syn::parse2(start).unwrap();
-        let generated = module.generate_swift();
+        let generated = module.generate_swift(&CodegenConfig::no_features_enabled());
 
         let expected = r#"
 @_cdecl("__swift_bridge__$some_function")
